import { Policy } from '../types.ts';

/** Policy options for `sizeLimitPolicy`. */
interface SizeLimitOptions {
  /** Maximum size of the message content in bytes. Default: 12KB (12288 bytes) */
  maxContentSize?: number;
  /** List of excluded event kinds */
  excludeKinds?: number[];
}

/**
 * Reject events larger than a specified size.
 *
 * @example
 * ```ts
 * // Reject events larger than a custom size (15KB) and exclude event kinds [1, 2].
 * sizeLimitPolicy(msg, { maxContentSize: 15000, excludeKinds: [1, 2] });
 * // Reject events larger than the default size (12KB) and exclude event kinds [3].
 * sizeLimitPolicy(msg, { excludeKinds: [3] });
 * ```
 */
const sizeLimitPolicy: Policy<SizeLimitOptions> = ({ event: { id, content, kind } }, opts = {}) => {
  const {
    excludeKinds = [3, 30078], // Exclude large events such as contact lists and app data
    maxContentSize = 12 * 1024  // Default size limit is 12KB
  } = opts;

  // Convert the content into bytes and check its size
  const contentSize = new TextEncoder().encode(content).length;

  if (contentSize > maxContentSize && !excludeKinds.includes(kind)) {
    const timestamp = new Date().toISOString();
    const logMessage = `${timestamp}: ID: ${id}, Kind: ${kind} rejected, ${contentSize} > max content size (${maxContentSize})\n`;
    Deno.writeTextFileSync('./size-limit-policy.log', logMessage, { append: true });
    return {
      id,
      action: 'reject',
      msg: `blocked: message is too large.`,
      //msg: `blocked: ${timestamp}: ID: ${id}, Kind: ${kind} rejected, ${contentSize} > max content size (${maxContentSize})\n`,
    };
  }

  return {
    id,
    action: 'accept',
    msg: '',
  };
};

export default sizeLimitPolicy;
